
#ifndef KORNERS_PROTOCOL
#define KORNERS_PROTOCOL

#include <QVector>
#include <QVariant>


class KornersMessage
{
public:
    enum MessageType
    {
        handshake_s = 1,
        handshake_c = 2,
        move = 3,
        regret = 4,
    };

    int type;
    QVector<QVariant> data;
};

class KornersProtocol : public QObject
{
    Q_OBJECT
public:
    KornersProtocol( QObject* parent = 0 ) : QObject( parent ) {}
    virtual ~KornersProtocol() {}
    virtual void processMessage( const QByteArray& ) = 0;
    virtual const QByteArray& start() = 0;
    virtual const QByteArray& stop() = 0;
    virtual const QByteArray& move( const QPoint& src, const QPoint& dst ) = 0;
public slots:
signals:
    void messageParsed( KornersMessage mess );
};


class KornersGameProtocol : public KornersProtocol
{
    Q_OBJECT
public:
    explicit KornersGameProtocol( const QString& name, bool server );
    ~KornersGameProtocol() {}
    void processMessage( const QByteArray& );
    const QByteArray& move( const QPoint& src, const QPoint& dst );
    const QByteArray& start();
    const QByteArray& stop();

private:
    int m_version;
    bool m_server;
    QString m_playerName;
};

#endif